package internal

import (
	"time"

	"github.com/name5566/leaf/gate"
	"github.com/name5566/leaf/log"
)

func init() {
	skeleton.RegisterChanRPC("NewAgent", rpcNewAgent)
	skeleton.RegisterChanRPC("CloseAgent", rpcCloseAgent)
}

const (
	ConnStatusUnauth = iota // 未认证
	ConnStatusAuthed        // 已认证
)

// 未认证连接管理
type PendingConn struct {
	Agent      gate.Agent
	CreateTime int64
	Status     int
}

var (
	// 未认证连接池
	pendingConns = make(map[gate.Agent]*PendingConn)
	// 已认证用户
	authedUsers = make(map[int64]gate.Agent) // key: userID
)

func rpcNewAgent(args []interface{}) {
	agent := args[0].(gate.Agent)
	// 将新连接放入未认证池
	pendingConns[agent] = &PendingConn{
		Agent:      agent,
		CreateTime: time.Now().Unix(),
		Status:     ConnStatusUnauth,
	}
	go checkTimeout(agent)
}

func rpcCloseAgent(args []interface{}) {
	agent := args[0].(gate.Agent)
	// 清理未认证连接
	_, exists := pendingConns[agent]
	if exists {
		delete(pendingConns, agent)
	}
	// 清理已认证连接
	for userID, a := range authedUsers {
		if a == agent {
			delete(authedUsers, userID)
			break
		}
	}
}

// 处理认证消息
func handleAuth(agent gate.Agent, userID int64) {
	// 找到未认证连接
	if pending, exists := pendingConns[agent]; exists {
		// 认证成功，移动到已认证池
		delete(pendingConns, agent)
		authedUsers[userID] = agent
		pending.Status = ConnStatusAuthed

		// 处理可能的重复登录
		if oldAgent, exists := authedUsers[userID]; exists && oldAgent != agent {
			oldAgent.Close()
			delete(authedUsers, userID)
		}
	}
}

// 定时检查超时连接
func checkTimeout(agent gate.Agent) {
	timeout := 30 * time.Second
	timer := time.NewTimer(timeout)
	defer timer.Stop()

	select {
	case <-timer.C:
		// 检查是否仍在未认证池中
		if conn, exists := pendingConns[agent]; exists {
			if conn.Status == ConnStatusUnauth {
				// 超时未认证，关闭连接
				log.Debug("Connection timeout without auth, closing...")
				agent.Close()
				delete(pendingConns, agent)
			}
		}
	}
}
